/*
 Copyright (c) 2009-2010 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
*/

#include <QDockWidget>
#include <QGridLayout>
#include <QFormLayout>
#include <QUrl>

#include <KActionCollection>
#include <KStandardAction>
#include <KApplication>
#include <KLocalizedString>
#include <KIcon>
#include <KAction>
#include <KLineEdit>
#include <KDebug>
#include <KConfigDialog>

#include "mainwindow.h"
#include "tab.h"
#include "placespanel.h"
#include "facetspanel.h"
#include "settings.h"
#include "facetsettings.h"

class MainWindow::Private
{
    public:
        Private(): tabWidget(0), placesWidget(0), facetsWidget(0), addressLine(0), panel(0), panel2(0)
        {
        }
        KTabWidget* tabWidget;
        QDockWidget* placesWidget;
        QDockWidget* facetsWidget;
        KLineEdit* addressLine;
        FacetsPanel* panel;
        PlacesPanel* panel2;
        KActionCollection* collection;
};

MainWindow::MainWindow ( QWidget* parent )
    : KXmlGuiWindow ( parent ),
      d(new Private())
{
    d->collection = new KActionCollection(qobject_cast<QWidget*>(this));
    
    // default actions
    KStandardAction::forward(this, SLOT(forward()), actionCollection());
    KStandardAction::back(this, SLOT(back()), actionCollection());
    KStandardAction::home(this, SLOT(home()), actionCollection());
    KStandardAction::selectAll(this, SLOT(selectAll()), actionCollection());
    KStandardAction::deselect(this, SLOT(deselect()), actionCollection());
    KStandardAction::quit(kapp, SLOT(quit()), actionCollection());
    KStandardAction::cut(this, SLOT(cut()), actionCollection());
    KStandardAction::copy(this, SLOT(copy()), actionCollection());
    KStandardAction::paste(this, SLOT(paste()), actionCollection());
    KStandardAction::preferences(this, SLOT(showPreferences()), actionCollection());

    // personalized actions
    KAction* newTabAction = new KAction(this);
    newTabAction->setText(i18n("New tab"));
    newTabAction->setIcon(KIcon("tab-new"));
    newTabAction->setShortcut(Qt::CTRL + Qt::Key_T);
    actionCollection()->addAction("newTab", newTabAction);
    connect(newTabAction, SIGNAL(triggered(bool)), this, SLOT(openNewTab()));
    KAction* iconsViewAction = new KAction(this);
    iconsViewAction->setText(i18n("Icons view"));
    iconsViewAction->setIcon(KIcon("view-list-icons"));
    actionCollection()->addAction("iconsView", iconsViewAction);
    connect(iconsViewAction, SIGNAL(triggered(bool)), this, SLOT(setIconsView()));
    KAction* detailedViewAction = new KAction(this);
    detailedViewAction->setText(i18n("Detailed view"));
    detailedViewAction->setIcon(KIcon("view-list-details"));
    actionCollection()->addAction("detailedView", detailedViewAction);
    connect(detailedViewAction, SIGNAL(triggered(bool)), this, SLOT(setDetailedView()));
    KAction* showPlaces = new KAction(this);
    showPlaces->setText(i18n("Show places"));
    actionCollection()->addAction("showPlaces", showPlaces);
    connect(showPlaces, SIGNAL(triggered(bool)), this, SLOT(showPlacesPanel()));
    KAction* showFacets = new KAction(this);
    showFacets->setText(i18n("Show facets"));
    actionCollection()->addAction("showFacets", showFacets);
    connect(showFacets, SIGNAL(triggered(bool)), this, SLOT(showFacetsPanel()));
    KAction* refresh = new KAction(KIcon("view-refresh"), i18n("Refresh"), this);
    actionCollection()->addAction("refresh", refresh);
    connect(refresh, SIGNAL(triggered(bool)), this, SLOT(refresh()));

    d->collection->readSettings();
    
    // init UI
    QWidget* container = new QWidget(this);
    QGridLayout* layout = new QGridLayout(container);
    QWidget* form = new QWidget(container);
    QFormLayout* flayout = new QFormLayout(form);
    d->addressLine = new KLineEdit(form);
    connect(d->addressLine, SIGNAL(returnPressed(const QString&)), this, SLOT(urlChanged(const QString&)));
    flayout->addRow(i18n("Location"), d->addressLine);
    d->tabWidget = new KTabWidget(container);
    d->tabWidget->setTabBarHidden(false);
    d->tabWidget->setTabsClosable(false);
    connect(d->tabWidget, SIGNAL(closeRequest(QWidget*)), this, SLOT(closeRequested(QWidget*)));
    layout->addWidget(form, 0, 0);
    layout->addWidget(d->tabWidget, 1, 0);
    setCentralWidget(container);
    setupGUI();

    // docks
    showPlacesPanel();
    showFacetsPanel();

    // misc
    connect(kapp, SIGNAL(aboutToQuit()), this, SLOT(quitting()));
    openNewTab();
    d->tabWidget->setFocus(Qt::OtherFocusReason);
}

MainWindow::~MainWindow()
{
    Settings::self()->writeConfig();
    delete d;
}

void MainWindow::back()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->back();
}

void MainWindow::forward()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->forward();
}

void MainWindow::home()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->home();
}

void MainWindow::refresh()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->redisplay();
}

void MainWindow::cut()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->cutCurrentSelection();
}

void MainWindow::copy()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->copyCurrentSelection();   
}

void MainWindow::paste()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->paste();   
}

void MainWindow::setIconsView()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->setIconsView();
}

void MainWindow::setDetailedView()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->setDetailedView();
}

void MainWindow::showPlacesPanel()
{
    if (!d->placesWidget) {
        d->placesWidget = new QDockWidget(i18n("Places"), this);
        d->placesWidget->setObjectName("placesWidget");
        d->placesWidget->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable);
        d->placesWidget->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
        d->panel2 = new PlacesPanel(d->placesWidget);
        d->placesWidget->setWidget(d->panel2);
        connect(d->panel2, SIGNAL(urlChanged(const KUrl&)), this, SLOT(placesUrlChanged(const KUrl&)));
        addDockWidget(Qt::LeftDockWidgetArea, d->placesWidget);
    }
    if (!d->placesWidget->isVisible()) {
        d->placesWidget->setVisible(true);
        addDockWidget(Qt::LeftDockWidgetArea, d->placesWidget);
    }
}

void MainWindow::showFacetsPanel()
{
    if (!d->facetsWidget) {
        d->facetsWidget = new QDockWidget(i18n("Facets"), this);
        d->facetsWidget->setObjectName("facetsWidget");
        d->facetsWidget->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable);
        d->facetsWidget->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
        d->panel = new FacetsPanel(Settings::facet().split(";"), d->facetsWidget);
        connect(d->facetsWidget, SIGNAL(visibilityChanged(bool)), d->panel, SLOT(dockVisible(bool)));
        connect(d->panel2, SIGNAL(urlChanged(const KUrl&)), d->panel, SLOT(dirChanged(const KUrl&)));
        connect(d->panel, SIGNAL(facetsDirChanged(const KUrl&)), this, SLOT(facetsUrlChanged(const KUrl&)));
        d->facetsWidget->setWidget(d->panel);
        addDockWidget(Qt::RightDockWidgetArea, d->facetsWidget);
    }
    if (!d->facetsWidget->isVisible()) {
        d->facetsWidget->setVisible(true);
        addDockWidget(Qt::RightDockWidgetArea, d->facetsWidget);
    }
}

void MainWindow::openNewTab()
{
    Tab* tab = new Tab();
    connect(tab, SIGNAL(dirChanged(const KUrl&)), this, SLOT(urlChanged(const KUrl&)));
    connect(tab, SIGNAL(dirChanged(const QString&)), d->addressLine, SLOT(setText(const QString&)));
    connect(tab, SIGNAL(dirChanged(const QString&)), d->panel, SLOT(dirChanged(const QString&)));
    int idx = d->tabWidget->addTab(tab, trailingDir(tab->address()));
    d->tabWidget->setCurrentIndex(idx);
    if (d->tabWidget->count() > 1) {
        d->tabWidget->setTabsClosable(true);
    }
    placesUrlChanged(KUrl(QDir::toNativeSeparators(QDir::homePath())));
    d->panel->dirChanged(KUrl(QDir::toNativeSeparators(QDir::homePath())));
}

void MainWindow::placesUrlChanged(const KUrl& url)
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->setAddress(url);
    d->tabWidget->setTabText(d->tabWidget->currentIndex(), trailingDir(url));
    d->addressLine->setText(url.path());
}

void MainWindow::urlChanged(const KUrl& url)
{
    d->tabWidget->setTabText(d->tabWidget->currentIndex(), trailingDir(url));
}

void MainWindow::urlChanged(const QString& path)
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->setAddress(KUrl(path));
    d->tabWidget->setTabText(d->tabWidget->currentIndex(), trailingDir(KUrl(path)));
}

void MainWindow::facetsUrlChanged(const KUrl& url)
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->setAddress(url);
    d->tabWidget->setTabText(d->tabWidget->currentIndex(), "Query");
    // this thing below should be changed in the end
    // but for debugging purposes is fine
    d->addressLine->setText(QUrl::fromPercentEncoding(url.toEncoded()));
}

void MainWindow::closeRequested(QWidget* sender)
{
    int idx = d->tabWidget->indexOf(sender);
    if (idx > -1) {
        d->tabWidget->removeTab(idx);
    }
    if (d->tabWidget->count() == 1) {
        d->tabWidget->setTabsClosable(false);
    }
}

void MainWindow::quitting()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->saveState();
}

void MainWindow::selectAll()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->selectAll();
}

void MainWindow::deselect()
{
    qobject_cast< Tab* >(d->tabWidget->currentWidget())->deselect();
}

void MainWindow::showPreferences()
{
    if (KConfigDialog::showDialog("settings")) {
        return;
    }
    KConfigDialog* settingsDialog = new KConfigDialog(this, "settings", Settings::self());
    settingsDialog->setFaceType(KPageDialog::List);
    connect(settingsDialog, SIGNAL(settingsChanged(const QString&)), this, SLOT(applySettings()));

    FacetSettings* facetSettings = new FacetSettings(settingsDialog);
    settingsDialog->addPage(facetSettings, i18nc("@title Preferences page name", "Facets"), "preferences-desktop-theme");
    connect(facetSettings, SIGNAL(settingsChanged()), this, SLOT(applySettings()));
    /* Unused (at least for now)
    connect(settingsDialog, SIGNAL(hidden()), this, SLOT(activate()));
    connect(settingsDialog, SIGNAL(closeClicked()), facetSettings, SLOT(resetSelection()));
    connect(settingsDialog, SIGNAL(cancelClicked()), facetSettings, SLOT(resetSelection()));*/
    settingsDialog->show();
}

QString MainWindow::trailingDir(const KUrl& url) const
{
    QString name;
    if (url.equals(KUrl("file:///"))) {
        name = '/';
    } else {
        name = url.fileName();
        if (name.isEmpty()) {
            name = url.protocol();
        } else {
            // Make sure that a '&' inside the directory name is displayed correctly
            // and not misinterpreted as a keyboard shortcut in QTabBar::setTabText()
            name.replace('&', "&&");
        }
    }
    return name;
}

void MainWindow::applySettings()
{
    // no on-the-fly changes, for now
}

#include "mainwindow.moc"
