/*
 Copyright (c) 2009-2010 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
*/

#include <QList>
#include <KDebug>

#include "facetmodel.h"
#include "term.h"

class FacetModel::Private
{
    public:
        Private()
        {
        }

        int find(const QString& name)
        {
            for (int i = 0; i < terms.size(); ++i) {
                if (terms[i].getName() == name) {
                    return i;
                }
            }

            return -1;
        }

        void clear()
        {
            QList<Term> tmp;
            foreach (Term t, terms) {
                if (t.isChecked()) {
                    tmp.append(t);
                }
            }
            terms.clear();
            terms.append(tmp);
            normalize();
        }

        /*
         * Important: if there are more than one checked item,
         * the last checked one has the count equal to the number
         * of files currently shown, while the old ones have the
         * old count; so here we are taking the minimum value and
         * then normalize all the other ones with it.
         */
        void normalize()
        {
            int m = min();
            foreach (Term t, terms) {
                if (t.getCount() > m) {
                    t.setCount(m);
                }
            }
        }

        /*
         * Calculated between checked items _only_.
         */
        int min()
        {
            // exagerated value, so it should always be greater than
            // the real min
            int min = 100000;

            foreach (Term t, terms) {
                if (t.isChecked() && t.getCount() < min) {
                    min = t.getCount();
                }
            }

            return min;
        }

        QList<Term> terms;
};

FacetModel::FacetModel(QObject* parent): QAbstractListModel(parent), d(new Private())
{
}

FacetModel::~FacetModel()
{
    delete d;
}

int FacetModel::rowCount(const QModelIndex& parent) const
{
    Q_UNUSED(parent);

    return d->terms.size();
}

QVariant FacetModel::data(const QModelIndex& index, int role) const
{
    QString text;

    if (index.row() < rowCount() && index.column() == 0)
    {
        switch (role) {
            case Qt::DisplayRole:
                text.append(d->terms[index.row()].getName());
                text.append(" (");
                text.append(QString::number(d->terms[index.row()].getCount()));
                text.append(")");

                return QVariant(text);
            case Qt::CheckStateRole:
                return QVariant(d->terms[index.row()].isChecked() ? Qt::Checked : Qt::Unchecked);
            case FacetModel::CountRole:
                return QVariant(d->terms[index.row()].getCount());
            case FacetModel::FrequencyRole:
                return QVariant(d->terms[index.row()].getFrequency());
        }
    }
    return QVariant();
}

Qt::ItemFlags FacetModel::flags(const QModelIndex& index) const
{
    if (index.isValid()) {
        return Qt::ItemIsEnabled | Qt::ItemIsUserCheckable;
    }
    else {
        return Qt::NoItemFlags;
    }
}

bool FacetModel::setData(const QModelIndex& index, const QVariant& value, int role)
{
    QString name;

    if (index.row() < rowCount() && index.column() == 0)
    {
        switch (role) {
            case Qt::CheckStateRole:
                name = d->terms[index.row()].getName();
                if (value.toInt() == Qt::Checked) {
                    d->terms[index.row()].setChecked(true);
                    emit itemChecked(name);
                }
                else {
                    d->terms[index.row()].setChecked(false);
                    emit itemUnchecked(name);
                }
                emit dataChanged(index, index);

                return true;
            case FacetModel::FrequencyRole:
                d->terms[index.row()].setFrequency(value.toInt());
                emit dataChanged(index, index);

                return true;
        }
    }

    return false;
}

void FacetModel::addNewItem(const QString& name, const int count)
{
    Term term(name, count, 0);
    int min = minCount();
    if (count > min) {
        term.setCount(min);
    }
    if (!d->terms.contains(term)) {
        beginInsertRows(QModelIndex(), rowCount(), rowCount());
        d->terms.append(term);
        endInsertRows();
    }
}

void FacetModel::updateFrequency(const QString& label, const int frequency)
{
    int i = d->find(label);
    if (i != -1) {
        QModelIndex idx = createIndex(i, 0);
        setData(idx, QVariant(frequency), FacetModel::FrequencyRole);
    }
}

void FacetModel::clear()
{
    beginResetModel();
    d->clear();
    endResetModel();
}

bool FacetModel::contains(const QString& tag) const
{
    return d->terms.contains(Term(tag, 0, 0));
}

int FacetModel::minCount() const
{
    return d->min();
}

#include "facetmodel.moc"
