/*
 Copyright (c) 2009-2010 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
*/

#include <QVBoxLayout>
#include <QList>
#include <QDir>
#include <KDebug>

#include <Soprano/Vocabulary/RDF>
#include <Nepomuk/Resource>
#include <Nepomuk/Query/AndTerm>
#include <Nepomuk/Query/ResourceTerm>
#include <Nepomuk/Query/ResourceTypeTerm>
#include <Nepomuk/Query/ComparisonTerm>
#include <Nepomuk/Query/NegationTerm>
#include <Nepomuk/Query/Query>
#include <Nepomuk/Query/FileQuery>

#include "facetspanel.h"
#include "daterange.h"
#include "nie.h"
#include "nfo.h"
#include "datefacet.h"
#include "typefacet.h"
#include "tagsfacet.h"

using namespace Nepomuk;

QStringList FacetsPanel::getFacetNames()
{
    QStringList result;

    result << "Date facet";
    result << "Tags facet";
    result << "Document type facet";

    return result;
}

class FacetsPanel::Private
{
    public:
        Private(): visible(true)
        {
        }

        bool visible;
        QString folder;
        QList<Facet*> facets;
};

FacetsPanel::FacetsPanel(QStringList facets, QWidget* parent): Panel(parent), d(new Private())
{
    QVBoxLayout* layout = new QVBoxLayout(this);

    if (facets.contains("Date facet")) {
        Facet* dateWidget = new DateFacet(this);
        connect(this, SIGNAL(folderChanged(const QString&)), dateWidget, SLOT(folderChanged(const QString&)));
        connect(this, SIGNAL(queryChanged()), dateWidget, SLOT(reload()));
        connect(dateWidget, SIGNAL(queryTermChanged(const Nepomuk::Query::Term&)), this, SLOT(updateQuery()));
        d->facets.append(dateWidget);
        layout->addWidget(dateWidget);
    }

    if (facets.contains("Document type facet")) {
        Facet* typeWidget = new TypeFacet(this);
        connect(this, SIGNAL(folderChanged(const QString&)), typeWidget, SLOT(folderChanged(const QString&)));
        connect(this, SIGNAL(queryChanged()), typeWidget, SLOT(reload()));
        connect(typeWidget, SIGNAL(queryTermChanged(const Nepomuk::Query::Term&)), this, SLOT(updateQuery()));
        d->facets.append(typeWidget);
        layout->addWidget(typeWidget);
    }

    if (facets.contains("Tags facet")) {
        Facet* tagsWidget = new TagsFacet(this);
        connect(this, SIGNAL(folderChanged(const QString&)), tagsWidget, SLOT(folderChanged(const QString&)));
        connect(this, SIGNAL(queryChanged()), tagsWidget, SLOT(reload()));
        connect(tagsWidget, SIGNAL(queryTermChanged(const Nepomuk::Query::Term&)), this, SLOT(updateQuery()));
        d->facets.append(tagsWidget);
        layout->addWidget(tagsWidget);
    }

    this->setLayout(layout);
}

FacetsPanel::~FacetsPanel()
{
    delete d;
}

void FacetsPanel::dirChanged(const QString& url)
{
    d->folder = url;
    if (d->visible) {
        emit folderChanged(url);
    }
}

void FacetsPanel::dirChanged(const KUrl& url)
{
    d->folder = url.path();
    if (d->visible) {
        emit folderChanged(url.path());
    }
}

void FacetsPanel::dockVisible(bool visible)
{
    if (visible != d->visible) {
        d->visible = visible;
    }
}

void FacetsPanel::updateQuery()
{
    // emit the query
    bool valid = false;
    for (int i = 0; i < d->facets.size(); ++i) {
        if (d->facets[i]->currentQueryTermValidation()) {
            valid = true;
            break;
        }
    }
    if (!valid) {
        emit facetsDirChanged(QDir::homePath());
    }
    else {
        emit facetsDirChanged(getQuery(getTerm()).toSearchUrl());
    }
    emit queryChanged();
}

Nepomuk::Query::Query FacetsPanel::getQuery(const Nepomuk::Query::Term& term) const
{
    Query::Query query;
    
    if (!term.isValid()) {
        query.setTerm(getTerm());
    }
    else {
        query.setTerm(term);
    }
    /*Query::FileQuery fileQuery(query);
    fileQuery.addIncludeFolder(KUrl(d->folder));*/
    
    return query;
}

Nepomuk::Query::Term FacetsPanel::getTerm() const
{
    // put together all the facets
    Query::AndTerm term;
    for (int i = 0; i < d->facets.size(); ++i) {
        if (d->facets[i]->currentQueryTermValidation()) {
            term.addSubTerm(d->facets[i]->currentQueryTerm());
        }
    }

    // if no valid term has been added, then emit the home directory
    if (!term.isValid()) {
        Query::ResourceTypeTerm cterm(Types::Class(Nepomuk::Vocabulary::NFO::FileDataObject()));
        term.addSubTerm(cterm);
    }

    // we always ignore folders
    Query::ComparisonTerm fterm(
            Types::Property(Soprano::Vocabulary::RDF::type()),
            Query::ResourceTerm(Nepomuk::Resource(Nepomuk::Vocabulary::NFO::Folder())),
            Query::ComparisonTerm::Equal);
    term.addSubTerm(Query::NegationTerm::negateTerm(fterm));
    
    return term;
}

QList<Facet*> FacetsPanel::getFacets() const
{
    return d->facets;
}

#include "facetspanel.moc"
