/*
 Copyright (c) 2009-2010 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
*/

#include <QButtonGroup>
#include <QToolButton>
#include <QVBoxLayout>
#include <QSizePolicy>
#include <QList>

#include <Nepomuk/Query/Term>
#include <Nepomuk/Query/ResourceTypeTerm>
#include <Nepomuk/Query/AndTerm>
#include <Nepomuk/Query/OrTerm>
#include <Nepomuk/Query/LiteralTerm>
#include <Nepomuk/Query/ComparisonTerm>
#include <Nepomuk/Types/Property>

#include "typefacet.h"
#include "typetoolbutton.h"
#include "nfo.h"
#include "nie.h"

using namespace Nepomuk;

namespace {
class TypeElement
{
    public:
        TypeElement(QString s, TypeToolButton::DocumentType t): text(s), type(t)
        {
        }
        QString text;
        TypeToolButton::DocumentType type;
};
}

class TypeFacet::Private
{
    public:
        Private(): any(0)
        {
        }
        QList<TypeElement*> terms;
        QToolButton* any;
        Query::Term queryTerm;
};

TypeFacet::TypeFacet(QWidget* parent): Facet(parent), d(new Private())
{
    setupUi(this);
    m_facetBox->setTitle("What?");
    QButtonGroup* group = new QButtonGroup(this);
    connect(group, SIGNAL(buttonClicked(QAbstractButton*)), this, SLOT(typeChanged(QAbstractButton*)));

    // terms
    d->terms.append(new TypeElement(i18n("All"), TypeToolButton::All));
    d->terms.append(new TypeElement(i18n("Documents"), TypeToolButton::Documents));
    d->terms.append(new TypeElement(i18n("Images"), TypeToolButton::Images));
    d->terms.append(new TypeElement(i18n("Media"), TypeToolButton::Media));

    QSizePolicy policy(QSizePolicy::Expanding, QSizePolicy::Minimum);
    for (int i = 0; i < d->terms.size(); ++i) {
        TypeToolButton* b = new TypeToolButton(this);
        b->setText(d->terms[i]->text);
        b->setType(d->terms[i]->type);
        b->setCheckable(true);
        if (i == 0) {
            b->setChecked(true);
        }
        b->setSizePolicy(policy);
        b->setAutoRaise(true);
        group->addButton(b);
        m_scrollLayout->addWidget(b, i, 0);
        if (i==0) {
            d->any = b;
        }
    }
}

TypeFacet::~TypeFacet()
{
    delete d;
}

void TypeFacet::doFolderChanged(const QString& folder)
{
    Q_UNUSED(folder);
    d->any->setChecked(true);
}

void TypeFacet::typeChanged(QAbstractButton* button)
{
    Query::OrTerm orTerm;
    TypeToolButton* b = qobject_cast<TypeToolButton*>(button);
    switch (b->getType()) {
        case TypeToolButton::All:
            // send and invalid term: the type query part is gone
            d->queryTerm = Query::Term();
            break;
        case TypeToolButton::Documents:
            d->queryTerm = Query::ResourceTypeTerm(Types::Class(Nepomuk::Vocabulary::NFO::Document()));
            break;
        case TypeToolButton::Media:
            orTerm.addSubTerm(Query::ResourceTypeTerm(Types::Class(Nepomuk::Vocabulary::NFO::Audio())));
            orTerm.addSubTerm(Query::ResourceTypeTerm(Types::Class(Nepomuk::Vocabulary::NFO::Video())));
            // need to check also the mimetype
            orTerm.addSubTerm(Query::ComparisonTerm(Nepomuk::Vocabulary::NIE::mimeType(), Query::LiteralTerm(QLatin1String("video"))));
            orTerm.addSubTerm(Query::ComparisonTerm(Nepomuk::Vocabulary::NIE::mimeType(), Query::LiteralTerm(QLatin1String("audio"))));
            d->queryTerm = orTerm;
            break;
        case TypeToolButton::Images:
            d->queryTerm = Query::ResourceTypeTerm(Types::Class(Nepomuk::Vocabulary::NFO::Image()));
            break;
        default:
            // not used
            break;
    }
    emit queryTermChanged(d->queryTerm);
}

Nepomuk::Query::Term TypeFacet::currentQueryTerm() const
{
    return d->queryTerm;
}

bool TypeFacet::currentQueryTermValidation() const
{
    return d->queryTerm.isValid();
}

#include "typefacet.moc"
