/*
 Copyright (c) 2009-2010 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
*/

#include <QGridLayout>
#include <QDebug>
#include <QAbstractItemView>
#include <QAbstractItemModel>
#include <QVariant>
#include <QMimeData>
#include <QApplication>
#include <QItemSelection>
#include <QClipboard>

#include <KDirOperator>
#include <KDirModel>
#include <KGlobal>
#include <KConfigGroup>
#include <KRun>
#include <KDirSortFilterProxyModel>
#include <KDebug>
#include <konqmimedata.h>
#include <konq_operations.h>

#include "tab.h"

class Tab::Private
{
    public:
        Private(): mode(KFile::Default), fileOp(0)
        {
        }
        KUrl address;
        KFile::FileView mode;
        KDirOperator* fileOp;
};

Tab::Tab(): QWidget(0), d(new Private())
{
    QGridLayout* layout = new QGridLayout(this);
    d->fileOp = new KDirOperator(d->address, this);
    loadState();
    layout->addWidget(d->fileOp);
    d->fileOp->setView(d->mode);
    d->fileOp->view()->setSelectionMode(QAbstractItemView::ExtendedSelection);
    connect(d->fileOp->view(), SIGNAL(clicked(const QModelIndex&)), this, SLOT(dirActivated(const QModelIndex&)));
    connect(d->fileOp, SIGNAL(fileSelected(const KFileItem&)), this, SLOT(fileSelected(const KFileItem&)));
}

Tab::~Tab()
{
    delete d;
}

void Tab::setAddress(const KUrl& address)
{
    d->address = address;
    d->fileOp->setUrl(d->address, true);
}

KUrl Tab::address()
{
    return d->address;
}

void Tab::setMode(KFile::FileView mode)
{
    d->mode = mode;
}

KFile::FileView Tab::mode()
{
    return d->mode;
}

void Tab::back()
{
    d->fileOp->back();
}

void Tab::forward()
{
    d->fileOp->forward();
}

void Tab::home()
{
    d->fileOp->home();
}

void Tab::redisplay()
{
    d->fileOp->rereadDir();
}

void Tab::setIconsView()
{
    d->fileOp->setView(KFile::PreviewContents);
}

void Tab::setDetailedView()
{
    d->fileOp->setView(KFile::Detail);
}

void Tab::saveState()
{
    KConfigGroup group(KGlobal::config(), "Tab view");
    d->fileOp->writeConfig(group);
}

void Tab::loadState()
{
    KConfigGroup group(KGlobal::config(), "Tab view");
    d->fileOp->readConfig(group);
}

void Tab::fileSelected(const KFileItem& item)
{
    new KRun(item.url(), 0);
}

void Tab::dirActivated(const QModelIndex& index)
{
    QAbstractItemView* view = d->fileOp->view();
    QAbstractItemModel* model = view->model();
    QVariant data = model->data(index, KDirModel::FileItemRole);
    KFileItem item = data.value<KFileItem>();
    if (item.isDir()) {
        emit dirChanged(item.url());
        emit dirChanged(item.url().path());
    }
}

void Tab::selectAll()
{
    d->fileOp->view()->selectAll();
}

void Tab::deselect()
{
    d->fileOp->view()->clearSelection();
}

void Tab::cutCurrentSelection()
{
    QMimeData* mimeData = getSelectedMime();
    if (mimeData != NULL) {
        KonqMimeData::addIsCutSelection(mimeData, true);
        QApplication::clipboard()->setMimeData(mimeData);
    }
}

void Tab::copyCurrentSelection()
{
    QMimeData* mimeData = getSelectedMime();
    if (mimeData != NULL) {
        QApplication::clipboard()->setMimeData(mimeData);
    }
}

void Tab::paste()
{
    // do not paste if we are browsing in a semantic way (even if some interesting modes could be applied here)
    if (QApplication::clipboard()->mimeData() != NULL && !d->address.protocol().startsWith("nepomuksearch")) {
        KonqOperations::doPaste(this, d->address);
    }
}

QMimeData* Tab::getSelectedMime() const
{
    KDirSortFilterProxyModel* proxyModel = qobject_cast<KDirSortFilterProxyModel*>(d->fileOp->view()->model());
    KDirModel* dirModel = qobject_cast<KDirModel*>(proxyModel->sourceModel());
    QItemSelectionModel* selModel = d->fileOp->view()->selectionModel();
    QItemSelection sel = proxyModel->mapSelectionToSource(selModel->selection());
    QModelIndexList list = sel.indexes();
    if (list.size() > 0) {
        return dirModel->mimeData(list);
    }
    else {
        return NULL;
    }
}

#include "tab.moc"
